import player
import math
from pygame import Rect
def getSwordLine(player): 
	point1 = -1
	point2 = -1
	height = -1
	if abs(player.getPlayerState("x_velocity")) < 10 and (player.getPlayerState("ghost_counter") > 300 or player.getPlayerState("ghost_counter") == -1): #if sword is ready
		if player.getDirection() == "left":
			if not player.getPlayerState("thrusting"): #if player is not attacking
				point1 = player.rect.x + 41
				point2 = player.rect.x + 97
			else: #player is attacking
				point1 = player.rect.x + 0
				point2 = player.rect.x + 55
		else:
			if not player.getPlayerState("thrusting"): #if player is not attacking
				point1 = player.rect.x + 171
				point2 = player.rect.x + 227
			else: #player is attacking
				point1 = player.rect.x + 213
				point2 = player.rect.x + 268
		if player.getPlayerState("sword_height") == 1:
			height = player.rect.y + 61
		elif player.getPlayerState("sword_height") == 2:
			height = player.rect.y + 37
		elif player.getPlayerState("sword_height") == 3:
			height = player.rect.y + 13
	return Rect(point1, height, point2-point1, 3)




import random
# TODO this will contain a list of every map and import them all
from redForestMap import RedForestMap


def selectMap(map_selection):
    if map_selection == 0:
        map_selection = random.randrange(1, 2, 1)  # update this function to include RNG chance of new map
    if map_selection == 1:
        return RedForestMap()



import gameEngine
import pygame
import sys
import menuButtons
import pauseButtons
import gameFrame


#To-do: create a function that checks for button sprite changes based on mx and my and an inputted string.

button_sprites = {
    "play_unhighl" : "Resources/Images/Buttons/PlayButtonUnhighlighted.png",
    "play_highl": "Resources/Images/Buttons/PlayButtonHighlighted.png",
    "play_clicked": "Resources/Images/Buttons/PlayButtonClicked.png",
    "exit_unhighl": "Resources/Images/Buttons/ExitButtonUnhighlighted.png",
    "exit_highl": "Resources/Images/Buttons/ExitButtonHighlighted.png",
    "exit_clicked": "Resources/Images/Buttons/ExitButtonClicked.png",
    "settings_unhighl": "Resources/Images/Buttons/SettingsButtonUnhighlighted.png",
    "settings_highl": "Resources/Images/Buttons/SettingsButtonHighlighted.png",
    "settings_clicked": "Resources/Images/Buttons/SettingsButtonClicked.png",
    "back_unhighl": "Resources/Images/Buttons/BackButtonUnhighlighted.png",
    "back_highl": "Resources/Images/Buttons/BackButtonHighlighted.png",
    "back_clicked": "Resources/Images/Buttons/BackButtonClicked.png",
    "restart_unhighl": "Resources/Images/Buttons/RestartButtonUnhighlighted.png",
    "restart_highl": "Resources/Images/Buttons/RestartButtonHighlighted.png",
    "restart_clicked": "Resources/Images/Buttons/RestartButtonClicked.png",
    "tutorial_unhighl": "Resources/Images/Buttons/TutorialButtonUnhighlighted.png",
    "tutorial_highl": "Resources/Images/Buttons/TutorialButtonHighlighted.png",
    "tutorial_clicked": "Resources/Images/Buttons/TutorialButtonClicked.png",
}

menu_buttons = {
    "play_button": menuButtons.Button(button_sprites["play_unhighl"], 400, 300, 220, 50),
    "settings_button": menuButtons.Button(button_sprites["settings_unhighl"], 400, 400, 220, 50),
    "exit_button": menuButtons.Button(button_sprites["exit_unhighl"], 400, 500, 220, 50),
    "back_button": menuButtons.Button(button_sprites["back_unhighl"], 400, 500, 220, 50),
    "restart_button": menuButtons.Button(button_sprites["restart_unhighl"], 0, 0, 220, 50),
    "tutorial_button": menuButtons.Button(button_sprites["tutorial_unhighl"], 0, 0, 220, 50)
}

#Remember to change game state booleans when going between menus.
game_state = {
    "start": False,
    "exit": False,
    "settings": False,
    "back": False,
    "paused": False,
    "resume": False,
    "restart": False,
    "tutorial": False
}

images_dictionary = dict()
pygame.font.init()
lobster_font = pygame.font.Font("Resources/Lobster.ttf", 64)


def mainMenu(screen):

    #Menu loop
    while True:
        screen.fill((0,0,0)) #Temporary black background
        mx, my = pygame.mouse.get_pos()

        m1_clicked = False
        #Event loop
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                if event.button == 1:
                    m1_clicked = True
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    pygame.quit()
                    sys.exit()


        #Button sprite changing with mouse interaction:
        if menu_buttons["play_button"].button_rect.collidepoint(mx, my):
            menu_buttons["play_button"].sprite = button_sprites["play_highl"]
            if m1_clicked: #Mouse click inside of the button's sprite
                game_state["start"] = True
                menu_buttons["play_button"].sprite = button_sprites["play_clicked"]
        else:
            menu_buttons["play_button"].sprite = button_sprites["play_unhighl"]

        if menu_buttons["exit_button"].button_rect.collidepoint(mx, my):
            menu_buttons["exit_button"].sprite = button_sprites["exit_highl"]
            if m1_clicked:
                game_state["exit"] = True
                menu_buttons["exit_button"].sprite = button_sprites["exit_clicked"]
        else:
            menu_buttons["exit_button"].sprite = button_sprites["exit_unhighl"]
        if menu_buttons["settings_button"].button_rect.collidepoint(mx, my):
            menu_buttons["settings_button"].sprite = button_sprites["settings_highl"]
            if m1_clicked:
                game_state["settings"] = True
                menu_buttons["settings_button"].sprite = button_sprites["settings_clicked"]
        else:
            menu_buttons["settings_button"].sprite = button_sprites["settings_unhighl"]

        if menu_buttons["tutorial_button"].button_rect.collidepoint(mx, my):
            menu_buttons["tutorial_button"].sprite = button_sprites["tutorial_highl"]
            if m1_clicked:
                game_state["tutorial"] = True
                menu_buttons["tutorial_button"].sprite = button_sprites["tutorial_clicked"]
        else:
            menu_buttons["tutorial_button"].sprite = button_sprites["tutorial_unhighl"]



        renderMenuButtons("Main", screen, menu_buttons)
        pygame.display.update()


        #Game state handling, i.e. changing menus, quitting, etc. in order to produce "button clicked" effect
        if game_state["start"]:
            pygame.time.delay(400)
            game_state["start"] = False #Reset game states to false.
            gameEngine.startGame(screen, 1, "Montoya", "Montoya")
        if game_state["tutorial"]:
            pygame.time.delay(400)
            game_state["tutorial"] = False
            tutorialMenu(screen)
        if game_state["settings"]:
            pygame.time.delay(400)
            game_state["settings"] = False
            settingsMenu(screen)
        if game_state["exit"]:
            pygame.time.delay(400)
            pygame.quit()
            sys.exit()

def tutorialMenu(screen):

    while True:
        screen.fill((0,0,0))
        mx, my = pygame.mouse.get_pos()

        m1_clicked = False
        #Event loop
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                if event.button == 1:
                    m1_clicked = True
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    game_state["back"] = True


        if menu_buttons["back_button"].button_rect.collidepoint(mx, my):
            menu_buttons["back_button"].sprite = button_sprites["back_highl"]
            if m1_clicked: #Mouse click inside of the button's sprite
                game_state["back"] = True
                menu_buttons["back_button"].sprite = button_sprites["back_clicked"]
        else:
            menu_buttons["back_button"].sprite = button_sprites["back_unhighl"]


        renderMenuButtons("Tutorial", screen, menu_buttons)
        pygame.display.update()

        if game_state["back"]:
            pygame.time.delay(400)
            game_state["back"] = False
            mainMenu(screen)

def settingsMenu(screen):

    #Menu loop
    while True:
        screen.fill((0,0,0))
        mx, my = pygame.mouse.get_pos()

        m1_clicked = False
        #Event loop
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                if event.button == 1:
                    m1_clicked = True
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    game_state["back"] = True


        if menu_buttons["back_button"].button_rect.collidepoint(mx, my):
            menu_buttons["back_button"].sprite = button_sprites["back_highl"]
            if m1_clicked: #Mouse click inside of the button's sprite
                game_state["back"] = True
                menu_buttons["back_button"].sprite = button_sprites["back_clicked"]
        else:
            menu_buttons["back_button"].sprite = button_sprites["back_unhighl"]


        renderMenuButtons("Settings", screen, menu_buttons)
        pygame.display.update()

        if game_state["back"]:
            pygame.time.delay(400)
            game_state["back"] = False
            if game_state["paused"]:
                pauseMenu(screen)
            else:
                mainMenu(screen)

def pauseMenu(screen, p1_meta_info, p2_meta_info, pause_buttons, draw_buffer, my_sprites): #Only can be brought up once in game

    #Make dirty sprite pause buttons visible
    pause_buttons["play_button"].visible = 1
    pause_buttons["restart_button"].visible = 1
    pause_buttons["exit_button"].visible = 1

    #Reset Player meta info to prevent infinite movement after unpausing.
    p1_meta_info["left"] = False
    p1_meta_info["right"] = False
    p1_meta_info["up"] = False
    p1_meta_info["down"] = False
    p2_meta_info["left"] = False
    p2_meta_info["right"] = False
    p2_meta_info["up"] = False
    p2_meta_info["down"] = False

    game_state["resume"] = False
    game_state["paused"] = True
    #Menu loop
    while True:
        mx, my = pygame.mouse.get_pos()

        m1_clicked = False
        #Event loop
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                if event.button == 1:
                    m1_clicked = True
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    game_state["resume"] = True


        if pause_buttons["play_button"].rect.collidepoint(mx, my):
            pause_buttons["play_button"].updateState("Highlighted")
            if m1_clicked: #Mouse click inside of the button's sprite
                game_state["resume"] = True
                pause_buttons["play_button"].updateState("Clicked")
        else:
            pause_buttons["play_button"].updateState("Unhighlighted")

        if pause_buttons["exit_button"].rect.collidepoint(mx, my):
            pause_buttons["exit_button"].updateState("Highlighted")
            if m1_clicked: #Mouse click inside of the button's sprite
                game_state["exit"] = True
                pause_buttons["exit_button"].updateState("Clicked")
        else:
            pause_buttons["exit_button"].updateState("Unhighlighted")

        if pause_buttons["restart_button"].rect.collidepoint(mx, my):
            pause_buttons["restart_button"].updateState("Highlighted")
            if m1_clicked:  # Mouse click inside of the button's sprite
                game_state["restart"] = True
                pause_buttons["restart_button"].updateState("Clicked")
        else:
            pause_buttons["restart_button"].updateState("Unhighlighted")

        pause_buttons["play_button"].update()
        pause_buttons["restart_button"].update()
        pause_buttons["exit_button"].update()

        gameFrame.render(my_sprites, draw_buffer)

        if game_state["resume"]:
            pygame.time.delay(400)
            game_state["paused"] = False
            game_state["resume"] = False
            pause_buttons["play_button"].visible = 0
            pause_buttons["restart_button"].visible = 0
            pause_buttons["exit_button"].visible = 0
            break

        if game_state["restart"]:
            pygame.time.delay(400)
            game_state["restart"] = False
            game_state["paused"] = False
            gameEngine.startGame(screen, 1, "Montoya", "Montoya")
        #if game_state["settings"]:
            #pygame.time.delay(400)
            #game_state["settings"] = False
            #game_state["paused"] = True
            #settingsMenu(screen)
        if game_state["exit"]:
            pygame.time.delay(400)
            game_state["exit"] = False
            game_state["paused"] = False
            mainMenu(screen)




#Reusable render function that will render depending on which menu loop you are in.
def renderMenuButtons(menu_name, screen, buttons):
    if menu_name == "Main":
        drawButton(buttons, screen, "play_button", 500, 330)
        drawButton(buttons, screen, "tutorial_button", 500, 400)
        drawButton(buttons,screen, "settings_button", 500, 470)
        drawButton(buttons, screen, "exit_button", 500, 540)
        #Draw the duel logo
        screen.blit(gameFrame.getImage("Resources/Images/MenuTitle.png", images_dictionary), (0,0))

    elif menu_name == "Settings":
        drawButton(buttons, screen, "back_button", 500, 450)

    elif menu_name == "Tutorial":
        #tutorial sprite draw
        screen.blit(gameFrame.getImage("Resources/Images/TutorialScreen.png", images_dictionary), (0,0))
        drawButton(buttons, screen, "back_button", 125, 560)

    #elif menu_name == "Pause":
        #drawButton(buttons, screen, "play_button", 500, 300)
        #drawButton(buttons, screen, "restart_button", 500, 350)
        #drawButton(buttons,screen, "settings_button", 500, 400)
        #drawButton(buttons, screen, "exit_button", 500, 400)



    #Add new menus by adding elif's and creating a function that holds the menu loop.

def drawButton(buttons, screen, button_name, x_pos, y_pos): #This function was not working when I implemented it, need to find a better way to simplify rendering function.
    #Center the button on its x and y coordinates to make it flush with the window.
    buttons[button_name].button_rect.center = (x_pos, y_pos)
    #Blit it to the screen
    screen.blit(gameFrame.getImage(buttons[button_name].sprite, images_dictionary), (buttons[button_name].button_rect.x, buttons[button_name].button_rect.y))


import pygame

class PauseButton(pygame.sprite.DirtySprite):

    def __init__(self, button_type):
        pygame.sprite.DirtySprite.__init__(self)
        self.button_state = {
            "unhighlighted": True,
            "highlighted": False,
            "clicked": False
        }
        self.button_type = button_type
        self.image = pygame.image.load(self.getSprite())
        self.rect = pygame.Rect(0,0, 220, 50)


    def update(self):
        self.image = pygame.image.load(self.getSprite())
        self.dirty = 1

    def getSprite(self):
        path = "Resources/Images/Buttons/"
        if self.button_state["unhighlighted"]:
            return path + self.button_type + "ButtonUnhighlighted.png"
        elif self.button_state["highlighted"]:
            return path + self.button_type + "ButtonHighlighted.png"
        elif self.button_state["clicked"]:
            return path + self.button_type + "ButtonClicked.png"

    def updateState(self, state):
        if state == "Highlighted":
            self.button_state["unhighlighted"] = False
            self.button_state["highlighted"] = True
            self.button_state["clicked"] = False
        elif state == "Clicked":
            self.button_state["unhighlighted"] = False
            self.button_state["highlighted"] = False
            self.button_state["clicked"] = True
        elif state == "Unhighlighted":
            self.button_state["unhighlighted"] = True
            self.button_state["highlighted"] = False
            self.button_state["clicked"] = False

    def initializeRect(self):
        if self.button_type == "Play":
            self.rect.center = (500,300)
        if self.button_type == "Restart":
            self.rect.center = (500, 350)
        if self.button_type == "Exit":
            self.rect.center = (500, 400)


"""
.. module:: map
.. synopsis: module for a map object
"""
from pygame import Rect
from abc import ABC, abstractmethod


class Map(ABC):
    """Class representing a General Map's attributes, this is applied to all maps, this is an abstract class
           :param x_length: the level's width
           :param y_length: the level's height
           :param p1_init_x_pos: the player's x-coordinate
           :param p1_init_y_pos: the player's y-coordinate
           :param p2_init_x_pos: the player's x-coordinate
           :param p2_init_y_pos: the player's y-coordinate
           :param background: The background image
           :param assets_dict: a dictionary of all image assets used in this map
           :param floor_height: a set height for the floor
        """

    def __init__(self, x_length, y_length, p1_init_x_pos, p1_init_y_pos, p2_init_x_pos, p2_init_y_pos, background, assets_dict):
        self._x_length = x_length
        self._y_length = y_length
        self._p1_init_pos = {
            "X": p1_init_x_pos,
            "Y": p1_init_y_pos
        }
        self._p2_init_pos = {
            "X": p2_init_x_pos,
            "Y": p2_init_y_pos
        }
        self._background = background
        self._assets_dict = assets_dict
        self._collidable_entities = []

    # abstract method
    ##@abstractmethod
    def setCollidableEntities(self):
        pass

    def getCollidableEntities(self):
        return self._collidable_entities

    def getXLength(self):
        return self._x_length

    def setXLength(self, length):
        self._x_length = length

    def getYLength(self):
        return self._y_length

    def setYLength(self, length):
        self._y_length = length

    def getP1InitialPosition(self):
        return self._p1_init_pos

    def setP1InitialPosition(self, x_pos, y_pos):
        self._p1_init_pos = {
            "X": x_pos,
            "Y": y_pos
            }

    def getP2InitialPosition(self):
        return self._p2_init_pos

    def setP2InitialPosition(self, x_pos, y_pos):
        self._p1_init_pos = {
            "X": x_pos,
            "Y": y_pos
        }

    def getBackground(self):
        return self._background

    def setBackground(self, new_background):
        self._background = new_background

    #collidable_entities = property(getCollidableEntities, setCollidableEntities)
    x_length = property(getXLength, setXLength)
    y_length = property(getYLength, setYLength)
    p1_init_pos = property(getP1InitialPosition, setP1InitialPosition)
    p2_init_pos = property(getP2InitialPosition, setP2InitialPosition)
    background = property(getBackground, setBackground)



import pygame
import os
from camera import Camera


def render(my_sprites, draw_buffer):
    rects = my_sprites.draw(draw_buffer)
    pygame.display.update(rects)  # copy rects from buffer to screen

def init(player1, player2, current_map, entities, pause_buttons):
    #camera = Camera(gameFrame.cameraMovement, current_map.x_length, current_map.y_length) # initializes camera with level's width and height
    imagesDict = dict()
    draw_buffer = pygame.display.set_mode((current_map.x_length, current_map.y_length))

    #create the background used to restore sprite previous location
    background = pygame.Surface(draw_buffer.get_size())
    pygame.draw.rect(background, (255, 255, 255, 255), entities[0])
    background.blit(getImage("Resources/Images/ScaledBackgroundAutumnForest.png", imagesDict), (0, 0))
    pygame.draw.rect(background, (139, 69, 19), entities[1])
    pygame.draw.rect(background, (139, 69, 19), entities[2])
    my_sprites = pygame.sprite.LayeredDirty()  # holds sprites to be drawn
    initializePauseButtons(pause_buttons)
    my_sprites.add(player1, player2, pause_buttons["play_button"], pause_buttons["restart_button"], pause_buttons["exit_button"])  # add both to our group
    my_sprites.clear(draw_buffer, background) # copy background to screen

    return draw_buffer, my_sprites

def cameraMovement(camera, target_rect): #method that determines centering on target
        l, t, _, _ = target_rect # l = left,  t = top
        _, _, w, h = camera     # w = width, h = height
        return pygame.Rect(-l+(.5 * camera.width), -t+(.5 * camera.height), w, h)

def getImage(path, _image_library):
    if path not in _image_library:
        canonicalized_path = path.replace('/', os.sep).replace('\\', os.sep)
        image = pygame.image.load(canonicalized_path).convert_alpha()
        _image_library[path] = image
    return _image_library[path]

def initializePauseButtons(pause_buttons):
    pause_buttons["play_button"].visible = 0
    pause_buttons["restart_button"].visible = 0
    pause_buttons["exit_button"].visible = 0
    pause_buttons["play_button"].initializeRect()
    pause_buttons["restart_button"].initializeRect()
    pause_buttons["exit_button"].initializeRect()


import pygame

#camera object allows for scrolling and centering on one player
class Camera:
    def __init__(self, cameraMovement, width, height):
        self.state = pygame.Rect(0, 0, width, height)
        self.cameraMovement = cameraMovement
		
    
	
    def apply(self, target):
	    return target.rect.move(self.state.topleft)

    def update(self, target):
	    self.state = self.cameraMovement(self.state, target.rect)
	
	
#Source: https://stackoverflow.com/questions/14354171/add-scrolling-to-a-platformer-in-pygame


"""
.. module:: Player
.. synopsis: module for a player object
"""
import pygame
from pygame import Rect
import time

# change these
hit_box_width = 102
hit_box_height = 140
image_shift_amount_x = 80  # This variable represents the amount of pixels of whitespace in the X dirrection from the left side of the player
image_shift_amount_y = 16  # This variable represents the amount of pixels of whitespace in the Y dirrection from the left side of the player


# 102x203
"""Grace's sprite update
        : no more x_pos and y_pos variables, now in self.rect.x and self.rect.y_pos
		: no more self.player_rect, uses self.rect instead
		: update updates the player's position, is called from the movement method
		: dx is the change in the player's x position
		: dy is the change in the player's y position

"""

class Player(pygame.sprite.DirtySprite):
    """Class representing individual players' avatar, their attributes, and movement
       :param direction_facing: the direction the player is facing (0 for left and 1 for right)
       :param sword_height: the height at which the player is holding the sword (0 means no sword, 3 is high, 2 is med, 1 is low sword)
       :param is_ghost: True if the player has respawned as a ghost and False if the player is not currently a ghost
       :param is_locked_on: True if camera is following this player, False otherwise
       :param sprite: the image name for the player's current motion
       :param image_dict: a dictionary of image file names for each motion
    """
    def __init__(self, x_pos, y_pos, direction_facing, sword_height, is_ghost, is_locked_on, image_dict):
        pygame.sprite.DirtySprite.__init__(self)
        self._player_state = {
            "running": False,
            "jumping": False,
            "ducking": False,
            "thrusting": False,
            "on_right_wall": False,
            "on_left_wall": False,
            "on_ground": False,
            "locked_on": False,
            "ghost": False,
            "ghost_counter": -1,
            "sword_height": 1,
            "direction_facing": direction_facing,
            "air_time": 0,
            "x_velocity": 0,
            "y_velocity": 0,
            "count_until_turn_around" :0
        }
        self._image_dict = image_dict
        self.image = pygame.image.load(self.getSprite()).convert_alpha()
        self.rect = Rect(x_pos + image_shift_amount_x, y_pos - image_shift_amount_y, 73, 140)

    def getXPos(self):
        return self.rect.x

    def getYPos(self):
        return self.rect.y

    def moveLeft(self):
        if (self.getPlayerState("ghost_counter") > 300 or self.getPlayerState("ghost_counter") == -1):
            self.setPlayerState("x_velocity", self.getPlayerState("x_velocity") - 1)
            if abs(self.getPlayerState("x_velocity")) > 5:
                self._player_state["count_until_turn_around"] += 1
            else:
                self._player_state["count_until_turn_around"] -= 4
            if self._player_state["count_until_turn_around"] > 15:
                self.setDirection("left")
                self._player_state["count_until_turn_around"] = 20
                self.setPlayerState("running", True)
            elif self._player_state["count_until_turn_around"] < 0:
                self._player_state["count_until_turn_around"] = 0
            if self.getPlayerState("x_velocity") < -10:
                self.setPlayerState("x_velocity", -10)
        else:
            self.setPlayerState("x_velocity", 0)

    def moveRight(self):
        if (self.getPlayerState("ghost_counter") > 300 or self.getPlayerState("ghost_counter") == -1):
            self.setPlayerState("x_velocity", self.getPlayerState("x_velocity") + 1)
            if abs(self.getPlayerState("x_velocity")) > 5:
                self._player_state["count_until_turn_around"] += 1
            else:
                self._player_state["count_until_turn_around"] -= 4
            if self._player_state["count_until_turn_around"] > 15:
                self.setDirection("right")
                self._player_state["count_until_turn_around"] = 20
                self.setPlayerState("running", True)
            elif self._player_state["count_until_turn_around"] < 0:
                self._player_state["count_until_turn_around"] = 0

            if self.getPlayerState("x_velocity") > 10:
                self.setPlayerState("x_velocity", 10)
        else:
            self.setPlayerState("x_velocity", 0)

    def standingStill(self):
        if self.getPlayerState("x_velocity") > 0:
            self.setPlayerState("x_velocity", self.getPlayerState("x_velocity") / 5)
        elif self.getPlayerState("x_velocity") < 0:
            self.setPlayerState("x_velocity", self.getPlayerState("x_velocity") / 5)
        if abs(self.getPlayerState("x_velocity")) < .1:
            self.setPlayerState("x_velocity", 0)
        if abs(self.getPlayerState("x_velocity")) < 1:
            self._player_state["count_until_turn_around"] -= 4
        if self._player_state["count_until_turn_around"] > 15:
            self.setDirection("right")
        elif self._player_state["count_until_turn_around"] < 0:
            self._player_state["count_until_turn_around"] = 0

    def calculateGravity(self, time):
        if self.getPlayerState("air_time") == 0:
            self.setPlayerState("air_time", time)
        self.setPlayerState("y_velocity", self.getPlayerState("y_velocity") + 5 * (time - self.getPlayerState("air_time")))
        if self.getPlayerState("y_velocity") > 50:
            self.setPlayerState("y_velocity", 50)
        if self.getPlayerState("on_ground"):
            self.setPlayerState("y_velocity", 0)

    def jump(self, time):
        self.setPlayerState("y_velocity", - 15)
        self.setPlayerState("air_time", time)
        self.setPlayerState("on_ground", False)

    def update(self):
        self.image = pygame.image.load(self.getSprite())
        self.dirty = 1  # force redraw from image, since we moved the sprite rect

    def getPlayerState(self, type):
        return self._player_state[type]

    def setPlayerState(self, type, value):
        self._player_state[type] = value

    def setDirection(self, direction):
        if (self.getPlayerState("ghost_counter") > 300 or self.getPlayerState("ghost_counter") == -1):
            if direction == "left":
                self._player_state["direction_facing"] = 1
            else:
                self._player_state["direction_facing"] = 0

    def getDirection(self):
        if self._player_state["direction_facing"] == 1:
            return "left"
        else:
            return "right"

    def adjustSwordHeight(self, adjustment):
        self._player_state["sword_height"] += adjustment
        if self._player_state["sword_height"] < 1:
            self._player_state["sword_height"] = 1
        elif self._player_state["sword_height"] > 3:
            self._player_state["sword_height"] = 3

    def respawn(self):
        if self._player_state["direction_facing"] == 1:
            self.rect.x += 50
        else:
            self.rect.x -= 50

    def getSprite(self):
        #print(self._player_state["ghost_counter"])
        if self._player_state["ghost"]:
            front = "ghost_"
        else:
            front = ""
        if self._player_state["direction_facing"] == 1:
            append = "_l"
        else:
            append = "_r"
        if self._player_state["ghost_counter"] >= 0 and self._player_state["ghost_counter"] < 100:
            return self._image_dict[front + "dead" + append + "_1"]
        elif self._player_state["ghost_counter"] >= 100 and self._player_state["ghost_counter"] < 200:
            return self._image_dict[front + "dead" + append + "_2"]
        elif self._player_state["ghost_counter"] >= 200 and self._player_state["ghost_counter"] < 300:    
            return self._image_dict[front + "dead" + append + "_3"]

        else:
            if self._player_state["ghost_counter"] == 301:
                self._player_state["ghost"] = True
            if self._player_state["running"]:
                return self._image_dict[front + "run" + append]
            if self._player_state["jumping"]:
                return self._image_dict[front + "jump" + append]
            if self._player_state["ducking"]:
                return self._image_dict[front + "duck" + append]
            if self._player_state["sword_height"] == 1:
                append = "_low"+append
            elif self._player_state["sword_height"] == 2:
                append = "_med" + append
            else:
                append = "_high" + append
            if self._player_state["thrusting"]:
                return self._image_dict[front + "thrust" + append]
            else:
                return self._image_dict[front + "sword" + append]

    def getImageDict(self):
        return self._image_dict

    def setImageDict(self, image_dict):
        self._image_dict = image_dict

    def getCollisionRect(self):
        return Rect(self.rect.x + image_shift_amount_x, self.rect.y - image_shift_amount_y, 73, 140)

    def debug(self):
        print("x_vel: ", end="")
        print(self.getPlayerState("x_velocity"), end=" ")
        print("y_vel: ", end="")
        print(self.getPlayerState("y_velocity"), end=" ")
        print("x_cord: ", end="")
        print(self.rect.x, end=" ")
        print("y_cord: ", end="")
        print(self.rect.y)
        print("count: ", end="")
        print(self.getPlayerState("count_until_turn_around"))

    def move(self, entities): #TODO Check for being stabbed in this method

        self.setPlayerState("on_ground", False)
        self.setPlayerState("on_right_wall", False)
        self.setPlayerState("on_left_wall", False)

        collision_list = self.test_collision_X(entities)  # Test all entities on the map for collision with player
        self.rect.x += (self.getPlayerState("x_velocity"))
        for objects in collision_list:
            if self.getPlayerState("x_velocity") < 0:  # Moving left
                self.rect.left = objects.right - image_shift_amount_x
                self.setPlayerState("on_left_wall", True)
            elif self.getPlayerState("x_velocity") > 0:  # Moving right
                self.rect.right = objects.left - image_shift_amount_x
                self.setPlayerState("on_right_wall", True)
        # Lock player to look at other player when standing still or moving short time.
        # Flip player if they have been moving a certain amount of time.

        collision_list = self.test_collision_Y(entities)
        self.rect.y += (self.getPlayerState("y_velocity"))
        for objects in collision_list:
            if self.getPlayerState("y_velocity") < 0:  # Moving up
                self.rect.top = objects.bottom - image_shift_amount_y
            elif self.getPlayerState("y_velocity") > 0:  # Moving down
                self.rect.bottom = objects.top + image_shift_amount_y
                self.setPlayerState("on_ground", True)
        self.update()  # updates players position

    def test_collision_Y(self, entities):
        collision_list = []
        self.rect.y += (self.getPlayerState("y_velocity"))
        for objects in entities:
            if self.getCollisionRect().colliderect(objects):
                collision_list.append(objects)
        self.rect.y -= (self.getPlayerState("y_velocity"))
        return collision_list

    def test_collision_X(self, entities):
        collision_list = []
        self.rect.x += (self.getPlayerState("x_velocity"))
        for objects in entities:
            if self.getCollisionRect().colliderect(objects):
                collision_list.append(objects)
        self.rect.x -= (self.getPlayerState("x_velocity"))
        return collision_list

    def duck(self):
        global hit_box_height
        hit_box_height /= 2

    def standUp(self):
        global hit_box_height
        hit_box_height *= 2

    sprite = property(getSprite)
    image_dict = property(getImageDict, setImageDict)
    duck = property(duck)
    stand_up = property(standUp)
    #Sources: https://github.com/gerryjenkinslb/pygame_dirtysprites/blob/master/Simple_Example_dirty_Sprites.py



"""
.. module:: map
.. synopsis: module for a map object
"""
from map import Map
from pygame import Rect


class RedForestMap(Map):
    """Class representing a Specific Map
        """
    def __init__(self):
        Map.__init__(self, 1000, 600, 450, 300, 550, 300, "Resources/Images/UF_Background.png", "assets_dict") #TODO: implement Assets Dict
        self.setCollidableEntities()

    def setCollidableEntities(self):
        floor_rect = Rect(-400, 500, 1600, 50)
        self._collidable_entities.append(floor_rect)
        block_rect = Rect(100, 400, 100, 100)
        self._collidable_entities.append(block_rect)
        block2_rect = Rect(800, 400, 100, 100)
        self._collidable_entities.append(block2_rect)

   # collidable_entities = property(getCollidableEntities, setCollidableEntities)


import pygame



class Button(pygame.sprite.Sprite):
    def __init__(self, sprite_path, x_pos, y_pos, rect_width, rect_height):
        pygame.sprite.Sprite.__init__(self)
        self.sprite = sprite_path
        self.button_rect = pygame.Rect(x_pos, y_pos, rect_width, rect_height)




"""
.. module:: gameEngine
.. synopsis: The Game's driving loop, This is where the whole game will be run from
"""
import mapSelectionList
import sys
from player import Player
import pygame
from pygame import Rect
from playerSkinsList import getSkin
import gameFrame
from swordHitBoxes import getSwordLine
from time import time
import mainMenuFrame
import pauseButtons
from camera import Camera
import math


# temp data, should not be here for long....
# Player 1 meta info, store inputs and send to the player object
p1_meta_info = {
    "up": False,  # if the player is moving in this direction
    "left": False,  # if the player is moving in this direction
    "right": False,  # if the player is moving in this direction
    "down": False,  # if the player is moving in this direction
    "sword_movement": 0, #if the sword is moving up its +1 if its moving down its -1 if not moving its 0
    "sword_down": False,  # down due to movement greater than 1/4sec
    "movement_clock": 0,  # time in milliseconds in which the player has moved without stopping
    "attack_count": 0
}

    # Player 2 meta info, store inputs and send to the player object
p2_meta_info = {
    "up": False,  # if the player is moving in this direction
    "left": False,  # if the player is moving in this direction
    "right": False,  # if the player is moving in this direction
    "down": False,  # if the player is moving in this direction
    "sword_movement": 0,  # if the sword is moving up its +1 if its moving down its -1 if not moving its 0
    "sword_down": False,  # down due to movement greater than 1/4sec
    "movement_clock": 0,  # time in milliseconds in which the player has moved without stopping
    "attack_count": 0
}

pause_buttons = {
    "play_button": pauseButtons.PauseButton("Play"),
    "restart_button": pauseButtons.PauseButton("Restart"),
    "exit_button": pauseButtons.PauseButton("Exit")
}

def mainMenu(screen):  # TODO Call Main Menu Frame instead and have it call startGame
    mainMenuFrame.mainMenu(screen)


def adjustPlayer(player, aspect, value):
    if player == 1:
        p1_meta_info[aspect] = value
    elif player == 2:
        p2_meta_info[aspect] = value

# :param Requires a Screen Objects (Created in the main passed to main menu),
#  an int map_selection to determine what map to put on,
# and 2 string skin_selection to determine what skins the players choose
def startGame(screen, map_selection, skin_selection1, skin_selection2):
    paused = False

    current_map = mapSelectionList.selectMap(map_selection)  # returns a child of the map class

    entities = current_map.getCollidableEntities()

    player1 = Player(-50, 100, 1, 2, False, True, getSkin(skin_selection1))  # Initializes player1
    player2 = Player(675, 100, 1, 2, False, False, getSkin(skin_selection2))  # Initializes player2

    draw_buffer, my_sprites = gameFrame.init(player1, player2, current_map, entities, pause_buttons)

    clock = pygame.time.Clock()

    #print("starting")

    player1_y_vel = 0 #temp will be altered soon
    player2_y_vel = 0  # temp will be altered soon

    active_match = True
    while active_match:
        # Delta time is implemented to help make sure that player's models will move at the same speed regardless of monitor refresh rate and processor speed.
        # Could use further optimizing and troubleshooting.
        clock.tick(90)

        for event in pygame.event.get():
            # What to do on quit
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()

            # Pressed a key so perform said action
            if event.type == pygame.KEYDOWN:

                if player1.getPlayerState("ghost_counter") == -1 or player1.getPlayerState("ghost_counter") > 300:
                    if event.key == pygame.K_d:
                        adjustPlayer(1, "right", True)
                    elif event.key == pygame.K_a:
                        adjustPlayer(1, "left", True)
                    if event.key == pygame.K_SPACE:
                        adjustPlayer(1, "up", True)
                    if event.key == pygame.K_s:
                        adjustPlayer(1, "sword_movement", -1)
                    elif event.key == pygame.K_w:
                        adjustPlayer(1, "sword_movement", 1)
                    if event.key == pygame.K_f:
                        adjustPlayer(1, "attack_count", 30)
                if player2.getPlayerState("ghost_counter") == -1 or player2.getPlayerState("ghost_counter") > 300:
                    if event.key == pygame.K_LEFT:
                        adjustPlayer(2, "left", True)
                    elif event.key == pygame.K_RIGHT:
                        adjustPlayer(2, "right", True)
                    if event.key == pygame.K_RSHIFT:
                        adjustPlayer(2, "up", True)
                    if event.key == pygame.K_DOWN:
                        adjustPlayer(2, "sword_movement", -1)
                    elif event.key == pygame.K_UP:
                        adjustPlayer(2, "sword_movement", 1)
                    if event.key == pygame.K_RCTRL:
                        adjustPlayer(2, "attack_count", 30)
                        player2.setPlayerState("attacking", True)
                if event.key == pygame.K_ESCAPE:
                    mainMenuFrame.pauseMenu(screen, p1_meta_info, p2_meta_info, pause_buttons, draw_buffer, my_sprites)


            # Let go of key so stop performing said action
            if event.type == pygame.KEYUP:
                if event.key == pygame.K_d:
                    adjustPlayer(1, "right", False)
                elif event.key == pygame.K_a:
                    adjustPlayer(1, "left", False)
                if event.key == pygame.K_SPACE:
                    adjustPlayer(1, "up", False)
                if event.key == pygame.K_LEFT:
                    adjustPlayer(2, "left", False)
                elif event.key == pygame.K_RIGHT:
                    adjustPlayer(2, "right", False)
                if event.key == pygame.K_RSHIFT:
                    adjustPlayer(2, "up", False)


        # NON EVENT BASED ACTIONS
        # Player 1 Sprite/Movement


        player1.setPlayerState("running", False)
        player2.setPlayerState("running", False)
        

        if p1_meta_info["attack_count"] > 0:
            player1.setPlayerState("thrusting", True)
            p1_meta_info["attack_count"] -= 1
        else:
            player1.setPlayerState("thrusting", False)
            if p1_meta_info["sword_movement"] != 0:
                player1.adjustSwordHeight(p1_meta_info["sword_movement"])
                p1_meta_info["sword_movement"] = 0

        if player1.rect.x > player2.rect.x:
            player1.setDirection("left")
        else:
            player1.setDirection("right")

        if p1_meta_info["left"]:
            player1.moveLeft()
        elif p1_meta_info["right"]:
            player1.moveRight()
        elif player1.getPlayerState("x_velocity") != 0:
            player1.standingStill()
        if p1_meta_info["up"] and player1.getPlayerState("on_ground") and (player1.getPlayerState("ghost_counter") > 300 or player1.getPlayerState("ghost_counter") == -1):
            player1.jump(time())
        elif not player1.getPlayerState("on_ground"):
            player1.calculateGravity(time())

        # Player 2 Sprite/Movement

        if p2_meta_info["attack_count"] > 0:
            player2.setPlayerState("thrusting", True)
            p2_meta_info["attack_count"] -= 1
        else :
            player2.setPlayerState("thrusting", False)
            if p2_meta_info["sword_movement"] != 0:
                player2.adjustSwordHeight(p2_meta_info["sword_movement"])
                p2_meta_info["sword_movement"] = 0

        if player2.rect.x > player1.rect.x:
            player2.setDirection("left")
        else:
            player2.setDirection("right")

        if p2_meta_info["left"]:
            player2.moveLeft()
        elif p2_meta_info["right"]:
            player2.moveRight()
        elif player2.getPlayerState("x_velocity") != 0:
            player2.standingStill()
        if p2_meta_info["up"] and player2.getPlayerState("on_ground") and (player2.getPlayerState("ghost_counter") > 300 or player2.getPlayerState("ghost_counter") == -1):
            player2.jump(time())
        elif not player2.getPlayerState("on_ground"):
            player2.calculateGravity(time())

        #Collision stuffs
        if player1.getPlayerState("ghost_counter") > 300 or player1.getPlayerState("ghost_counter") == -1:
            player1body = Rect(player1.rect.x + 127, player1.rect.y - 16, 15, 140)
        else:
            player1body = Rect(-100, -100, 1, 1)

        if player2.getPlayerState("ghost_counter") > 300 or player2.getPlayerState("ghost_counter") == -1:
            player2body = Rect(player2.rect.x + 127, player2.rect.y - 16, 15, 140)
        else:
            player2body = Rect(-100, -100, 1, 1)

        while getSwordLine(player1).colliderect(getSwordLine(player2)):
            #print("Clash!")
            player1.setPlayerState("x_velocity", 0)
            player2.setPlayerState("x_velocity", 0)
            if player1.getDirection() == "left":
                player1.moveRight()
                player2.moveLeft()
                player1.move(entities)
                player2.move(entities)
            else:
                player1.moveLeft()
                player2.moveRight()
                player1.move(entities)
                player2.move(entities)
        else:
            if player1body.colliderect(getSwordLine(player2)) and player2body.colliderect(getSwordLine(player1)):
                #print("players both died")
                player1.setPlayerState("ghost_counter", 0)
                player2.setPlayerState("ghost_counter", 0)
                

                #screen locked in place
            elif player1body.colliderect(getSwordLine(player2)):
                #print("player 1 had an ouchie")
                player1.setPlayerState("ghost_counter", 0) #Should start a counter for each frame of death animation, followed by a respawn delay, followed by drawing them as a ghost in that spot
                player2.setPlayerState("ghost_counter", -1)
                player2.setPlayerState("ghost", False)
                #screen follows player 2
            elif player2body.colliderect(getSwordLine(player1)):
                #print("player 2 had an ouchie")
                player2.setPlayerState("ghost_counter", 0) #Should start a counter for each frame of death animation, followed by a respawn delay, followed by drawing them as a ghost in that spot
                player1.setPlayerState("ghost_counter", -1)
                player1.setPlayerState("ghost", False)
                #screen follows player 1

        if player1.getPlayerState("ghost_counter") > -1:
            #print("woooooowie")
            player1.setPlayerState("ghost_counter", player1.getPlayerState("ghost_counter") + 1)
        if player2.getPlayerState("ghost_counter") > -1:
            #print("woooooowie 2")
            player2.setPlayerState("ghost_counter", player2.getPlayerState("ghost_counter") + 1)
        if player1.getPlayerState("ghost_counter") == 300:
            #print("woooooowie")
            player1.respawn()
        if player2.getPlayerState("ghost_counter") == 300:
            #print("woooooowie 2")
            player2.respawn()

        player1.move(entities)
        player2.move(entities)


        #render
        gameFrame.render(my_sprites, draw_buffer)

#if player1.getPlayerState("ghost_counter") >= 0 and player1.getPlayerState("ghost_counter") < 11:


# Skin Example
# skin = {"stand_l": "",
# "stand_r": "",
# "sword_high_l": "",
# "sword_high_r": "",
# "sword_med_l": "",
# "sword_med_r": "",
# "sword_low_l": "",
# "sword_low_r": "",
# "duck_l": "",
# "duck_r": "",
# "jump_l": "",
# "jump_r": "",
# "thrust_high_l": "",
# "thrust_high_r": "",
# "thrust_med_l": "",
# "thrust_med_r": "",
# "thrust_low_l": "",
# "thrust_low_r": ""}


def getSkin(character_type):
    images_dictionary = dict()
    if character_type == "Montoya":
        images_dictionary = {
            "run_l": "Resources/Images/MontoyaRunL.png",
            "run_r": "Resources/Images/MontoyaRunR.png",
            "sword_high_l": "Resources/Images/MontoyaHighL.png",
            "sword_high_r": "Resources/Images/MontoyaHighR.png",
            "sword_med_l": "Resources/Images/MontoyaMedL.png",
            "sword_med_r": "Resources/Images/MontoyaMedR.png",
            "sword_low_l": "Resources/Images/MontoyaLowL.png",
            "sword_low_r": "Resources/Images/MontoyaLowR.png",
            "duck_l": "Resources/Images/MontoyaDuckL.png",
            "duck_r": "Resources/Images/MontoyaDuckR.png",
            "thrust_high_l": "Resources/Images/MontoyaHighThrustL.png",
            "thrust_high_r": "Resources/Images/MontoyaHighThrustR.png",
            "thrust_med_l": "Resources/Images/MontoyaMedThrustL.png",
            "thrust_med_r": "Resources/Images/MontoyaMedThrustR.png",
            "thrust_low_l": "Resources/Images/MontoyaLowThrustL.png",
            "thrust_low_r": "Resources/Images/MontoyaLowThrustR.png",
            "dead_l_1": "Resources/Images/MontoyaDeath1L.png",
            "dead_r_1": "Resources/Images/MontoyaDeath1R.png",
            "dead_l_2": "Resources/Images/MontoyaDeath2L.png",
            "dead_r_2": "Resources/Images/MontoyaDeath2R.png",
            "dead_l_3": "Resources/Images/MontoyaDeath3L.png",
            "dead_r_3": "Resources/Images/MontoyaDeath3R.png",
            "ghost_run_l": "Resources/Images/MontoyaGhostRunL.png",
            "ghost_run_r": "Resources/Images/MontoyaGhostRunR.png",
            "ghost_sword_high_l": "Resources/Images/MontoyaGhostHighL.png",
            "ghost_sword_high_r": "Resources/Images/MontoyaGhostHighR.png",
            "ghost_sword_med_l": "Resources/Images/MontoyaGhostMedL.png",
            "ghost_sword_med_r": "Resources/Images/MontoyaGhostMedR.png",
            "ghost_sword_low_l": "Resources/Images/MontoyaGhostLowL.png",
            "ghost_sword_low_r": "Resources/Images/MontoyaGhostLowR.png",
            "ghost_duck_l": "Resources/Images/MontoyaGhostDuckL.png",
            "ghost_duck_r": "Resources/Images/MontoyaGhostDuckR.png",
            "ghost_thrust_high_l": "Resources/Images/MontoyaGhostHighThrustL.png",
            "ghost_thrust_high_r": "Resources/Images/MontoyaGhostHighThrustR.png",
            "ghost_thrust_med_l": "Resources/Images/MontoyaGhostMedThrustL.png",
            "ghost_thrust_med_r": "Resources/Images/MontoyaGhostMedThrustR.png",
            "ghost_thrust_low_l": "Resources/Images/MontoyaGhostLowThrustL.png",
            "ghost_thrust_low_r": "Resources/Images/MontoyaGhostLowThrustR.png",
            "ghost_dead_l_1": "Resources/Images/MontoyaGhostDeath1L.png",
            "ghost_dead_r_1": "Resources/Images/MontoyaGhostDeath1R.png",
            "ghost_dead_l_2": "Resources/Images/MontoyaGhostDeath2L.png",
            "ghost_dead_r_2": "Resources/Images/MontoyaGhostDeath2R.png",
            "ghost_dead_l_3": "Resources/Images/MontoyaGhostDeath3L.png",
            "ghost_dead_r_3": "Resources/Images/MontoyaGhostDeath3R.png"
        }
    else:
        # set a default skin
        images_dictionary = {
            "run_l": "Resources/Images/MontoyaRunL.png",
            "run_r": "Resources/Images/MontoyaRunR.png",
            "sword_high_l": "Resources/Images/MontoyaHighL.png",
            "sword_high_r": "Resources/Images/MontoyaHighR.png",
            "sword_med_l": "Resources/Images/MontoyaMedL.png",
            "sword_med_r": "Resources/Images/MontoyaMedR.png",
            "sword_low_l": "Resources/Images/MontoyaLowL.png",
            "sword_low_r": "Resources/Images/MontoyaLowR.png",
            "duck_l": "Resources/Images/MontoyaDuckL.png",
            "duck_r": "Resources/Images/MontoyaDuckR.png",
            "thrust_high_l": "Resources/Images/MontoyaHighThrustL.png",
            "thrust_high_r": "Resources/Images/MontoyaHighThrustR.png",
            "thrust_med_l": "Resources/Images/MontoyaMedThrustL.png",
            "thrust_med_r": "Resources/Images/MontoyaMedThrustR.png",
            "thrust_low_l": "Resources/Images/MontoyaLowThrustL.png",
            "thrust_low_r": "Resources/Images/MontoyaLowThrustR.png",
            "dead_l_1": "Resources/Images/MontoyaDeath1L.png",
            "dead_r_1": "Resources/Images/MontoyaDeath1R.png",
            "dead_l_2": "Resources/Images/MontoyaDeath2L.png",
            "dead_r_2": "Resources/Images/MontoyaDeath2R.png",
            "dead_l_3": "Resources/Images/MontoyaDeath3L.png",
            "dead_r_3": "Resources/Images/MontoyaDeath3R.png",
            "ghost_run_l": "Resources/Images/MontoyaGhostRunL.png",
            "ghost_run_r": "Resources/Images/MontoyaGhostRunR.png",
            "ghost_sword_high_l": "Resources/Images/MontoyaGhostHighL.png",
            "ghost_sword_high_r": "Resources/Images/MontoyaGhostHighR.png",
            "ghost_sword_med_l": "Resources/Images/MontoyaGhostMedL.png",
            "ghost_sword_med_r": "Resources/Images/MontoyaGhostMedR.png",
            "ghost_sword_low_l": "Resources/Images/MontoyaGhostLowL.png",
            "ghost_sword_low_r": "Resources/Images/MontoyaGhostLowR.png",
            "ghost_duck_l": "Resources/Images/MontoyaGhostDuckL.png",
            "ghost_duck_r": "Resources/Images/MontoyaGhostDuckR.png",
            "ghost_thrust_high_l": "Resources/Images/MontoyaGhostHighThrustL.png",
            "ghost_thrust_high_r": "Resources/Images/MontoyaGhostHighThrustR.png",
            "ghost_thrust_med_l": "Resources/Images/MontoyaGhostMedThrustL.png",
            "ghost_thrust_med_r": "Resources/Images/MontoyaGhostMedThrustR.png",
            "ghost_thrust_low_l": "Resources/Images/MontoyaGhostLowThrustL.png",
            "ghost_thrust_low_r": "Resources/Images/MontoyaGhostLowThrustR.png",
            "ghost_dead_l_1": "Resources/Images/MontoyaGhostDeath1L.png",
            "ghost_dead_r_1": "Resources/Images/MontoyaGhostDeath1R.png",
            "ghost_dead_l_2": "Resources/Images/MontoyaGhostDeath2L.png",
            "ghost_dead_r_2": "Resources/Images/MontoyaGhostDeath2R.png",
            "ghost_dead_l_3": "Resources/Images/MontoyaGhostDeath3L.png",
            "ghost_dead_r_3": "Resources/Images/MontoyaGhostDeath3R.png"
        }
    return images_dictionary
